(*************************************************************************

     $RCSfile: RKMButtonclass.mod $
  Description: Example Boopsi gadget for RKRM:Libraries

   Created by: fjc (Frank Copeland)
    $Revision: 1.1 $
      $Author: fjc $
        $Date: 1994/08/08 16:55:25 $

  Copyright  1994, Frank Copeland.
  This example program is part of Oberon-A.
  See Oberon-A.doc for conditions of use and distribution.

  Log entries are at the end of the file.

*************************************************************************)

MODULE RKMButtonclass;

(*
** $C= CaseChk       $I= IndexChk  $L= LongAdr   $N- NilChk
** $P- PortableCode  $R= RangeChk  $S- StackChk  $T= TypeChk
** $V= OvflChk       $Z= ZeroVars
*)

IMPORT
  SYS := SYSTEM,
  e   := Exec,
  i   := Intuition,
  u   := Utility,
  gfx := Graphics,
  bu  := BoopsiUtil,
  hu  := HookUtil,
  IE  := InputEvent,
  Errors,
  d   := Dos(*,
  iu := IntuiUtil*);

CONST
  VersionTag = "$VER: RKMButtonclass 1.0 (18.7.94)\n";
  VersionStr = "RKMButtonclass 1.0 (18 Jul 1994)\n";
  CopyrightStr = "Copyright  1994 Frank Copeland";

(*
** Class specifics
*)

CONST

  rkmButPulse = u.tagUser + 1;

TYPE

  ButINSTPtr = CPOINTER TO ButINST;
  ButINST = RECORD
    midX, midY : LONGINT; (* Co-ordinates of middle of gadget *)
  END;

CONST

(* ButINST has one flag: *)

  eraseOnly = 0; (* Tells rendering routine to *)
                 (* only erase the gadget, not *)
                 (* rerender a new one.  This  *)
                 (* lets the gadget erase it-  *)
                 (* self before it rescales.   *)

(**************************************************************************)
(* The Main procedure connects an RKMButClass object to a Boopsi integer  *)
(* gadget, which displays the RKMButClass gadget's rkmButPulse value.     *)
(* The code scales and moves the gadget while it is in place.             *)
(**************************************************************************)

VAR

  pulse2int : ARRAY 2 OF u.TagItem;

CONST

  intWidth = 40;
  intHeight = 20;

VAR

  w : i.WindowPtr;
  rkmbutcl : i.IClassPtr;
  integer, but : i.GadgetPtr;
  msg : i.IntuiMessagePtr;

(*------------------------------------*)
PROCEDURE^ freeRKMButGadClass ( cl : i.IClassPtr );

PROCEDURE* Cleanup;
BEGIN (* Cleanup *)
  IF but # NIL THEN
    SYS.PUTREG (0, i.base.RemoveGList (w, integer, -1));
    i.base.DisposeObject (but); but := NIL
  END;
  IF integer # NIL THEN i.base.DisposeObject (integer); integer := NIL END;
  IF rkmbutcl # NIL THEN freeRKMButGadClass (rkmbutcl); rkmbutcl := NIL END;
  IF w # NIL THEN i.base.CloseWindow (w); w := NIL END
END Cleanup;

(*------------------------------------*)
PROCEDURE Init ();

BEGIN (* Init *)
  u.OpenLib (TRUE);
  pulse2int [0].tag := rkmButPulse; pulse2int [0].data := i.stringaLongVal;
  pulse2int [1].tag := u.tagEnd;
  but := NIL; integer := NIL; rkmbutcl := NIL; w := NIL;
  SYS.SETCLEANUP (Cleanup)
END Init;


(*------------------------------------*)
PROCEDURE MainLoop ( attr, value : LONGINT );

  VAR done : BOOLEAN; ignore : LONGINT;

BEGIN (* MainLoop *)
  done := FALSE;
  ignore := i.base.SetGadgetAttrs (but^, w, NIL, attr, value, u.tagDone);
  WHILE ~done DO
    e.base.WaitPort (w.userPort);
    LOOP
      msg := SYS.VAL (i.IntuiMessagePtr, e.base.GetMsg (w.userPort));
      IF msg = NIL THEN EXIT END;
      IF msg.class = {i.idcmpCloseWindow} THEN done := TRUE END;
      e.base.ReplyMsg (msg)
    END;
  END;
END MainLoop;

(*------------------------------------*)
PROCEDURE RenderRKMBut
  ( cl : i.IClassPtr; g : i.GadgetPtr; msg : i.RenderPtr )
  : e.APTR;

  VAR
    inst : ButINSTPtr;
    rp : gfx.RastPortPtr;
    retval : e.APTR;
    pens : i.DRIPenArrayPtr;
    back, shine, shadow, wd, h, x, y : INTEGER;

BEGIN (* RenderRKMBut *)
  inst := bu.InstData (cl, SYS.VAL (i.ObjectPtr, g));
  retval := SYS.VAL (e.APTR, e.LTRUE);
  pens := msg.gInfo.drInfo.pens;
  IF msg.methodID = i.gmRender THEN (* If msg is truly a gmRender message *)
                                    (* (not a Input that looks like a     *)
                                    (* Render), use the rastport within   *)
                                    (* it...                              *)
    rp := msg.rPort
  ELSE                              (* ...Otherwise, get a rastport using *)
                                    (* ObtainGIRPort().                   *)
    rp := i.base.ObtainGIRPort (msg.gInfo)
  END;
  IF rp # NIL THEN
    IF i.gflgSelected IN g.flags THEN      (* If the gadget is selected,  *)
                                           (* reverse the meanings of the *)
                                           (* pens.                       *)
      back := pens [i.fillPen];
      shine := pens [i.shadowPen];
      shadow := pens [i.shinePen]
    ELSE
      back := pens [i.backGroundPen];
      shine := pens [i.shinePen];
      shadow := pens [i.shadowPen]
    END;
    gfx.base.SetDrMd (rp, gfx.jam1);

    gfx.base.SetAPen (rp, SHORT (back));          (* Erase the old gadget *)
    gfx.base.RectFill
      ( rp, g.leftEdge,
            g.topEdge,
            g.leftEdge + g.width,
            g.topEdge + g.height );

    gfx.base.SetAPen (rp, SHORT (shadow));            (* Draw shadow edge *)
    gfx.base.Move (rp, g.leftEdge + 1, g.topEdge + g.height);
    gfx.base.Draw (rp, g.leftEdge + g.width, g.topEdge + g.height);
    gfx.base.Draw (rp, g.leftEdge + g.width, g.topEdge + 1);

    wd := g.width DIV 4;         (* Draw arrows - Sorry, no frills imagery *)
    h := g.height DIV 2;
    x := g.leftEdge + (wd DIV 2);
    y := g.topEdge + (h DIV 2);

    gfx.base.Move (rp, x, SHORT (inst.midY));
    gfx.base.Draw (rp, x + wd, y);
    gfx.base.Draw (rp, x + wd, y + g.height - h);
    gfx.base.Draw (rp, x, SHORT (inst.midY));

    x := g.leftEdge + (wd DIV 2) + g.width DIV 2;

    gfx.base.Move (rp, x + wd, SHORT (inst.midY));
    gfx.base.Draw (rp, x, y);
    gfx.base.Draw (rp, x, y + g.height - h);
    gfx.base.Draw (rp, x + wd, SHORT (inst.midY));

    gfx.base.SetAPen (rp, SHORT (shine));              (* Draw shine edge *)
    gfx.base.Move (rp, g.leftEdge, g.topEdge + g.height - 1);
    gfx.base.Draw (rp, g.leftEdge, g.topEdge);
    gfx.base.Draw (rp, g.leftEdge + g.width - 1, g.topEdge);

    IF msg.methodID # i.gmRender THEN (* If we allocated a rastport, give *)
                                      (* it back. *)
      i.base.ReleaseGIRPort (rp)
    END;
  ELSE
    retval := SYS.VAL (e.APTR, e.LFALSE);
  END;
  RETURN retval
END RenderRKMBut;


(*------------------------------------*)
PROCEDURE NotifyPulse
  ( cl    : i.IClassPtr;
    o     : i.ObjectPtr;
    flags : SET;
    mid   : LONGINT;
    gpi   : i.InputPtr );

  VAR
    tt : ARRAY 3 OF u.TagItem;
    g : i.GadgetPtr;
    ignore : e.APTR;

BEGIN (* NotifyPulse *)
  g := SYS.VAL (i.GadgetPtr, o);

  tt[0].tag := rkmButPulse;
  tt[0].data := mid - gpi.mouse.x + g.leftEdge;

  tt[1].tag := i.gaID;
  tt[1].data := g.gadgetID;

  tt[2].tag := u.tagDone;

  ignore := bu.DoSuperMethod
    (cl, o, i.omNotify, SYS.ADR (tt), gpi.gInfo, flags)
END NotifyPulse;

(*------------------------------------*)
PROCEDURE* dispatchRKMButGad
  ( hook : u.HookPtr; obj : e.APTR; message : e.APTR )
  : e.APTR;

  VAR
    cl : i.IClassPtr; o : i.ObjectPtr; msg : i.MsgPtr;
    inst : ButINSTPtr;
    retval, ignore : SYS.LONGWORD;
    object : i.ObjectPtr;
    g : i.GadgetPtr;
    gpi : i.InputPtr;
    ie : IE.InputEventPtr;
    rp : gfx.RastPortPtr;
    x, y, wd, h : INTEGER;
    pens : i.DRIPenArrayPtr;
    opSet : i.OpSetPtr;

(* $s+ Save registers on entry *)
BEGIN (* dispatchRKMButGad *)
  (* SYS.INLINE (048E7H, 03F32H); *)
  cl := SYS.VAL (i.IClassPtr, hook);
  o := obj;
  msg := message;
  retval := e.LTRUE;
  CASE msg.methodID OF
    i.omNew : (* First, pass up to superclass *)
      object := bu.DoSuperMethodA (cl, o, msg^);
      IF object # NIL THEN
        g := SYS.VAL (i.GadgetPtr, object);
                (* Initial local instance data *)
        inst := bu.InstData (cl, object);
        inst.midX := g.leftEdge + (g.width DIV 2);
        inst.midY := g.topEdge + (g.height DIV 2);
        retval := object
      END;
    |
    i.gmHitTest :
          (* Since this is a rectangular gadget this *)
          (* method always returns i.gmrGadgetHit.   *)
      retval := i.gmrGadgetHit;
    |
    i.gmGoActive :
      inst := bu.InstData (cl, o);
          (* Only become active if the gmGoActive *)
          (* was triggered by direct user input.  *)
      gpi := SYS.VAL (i.InputPtr, msg);
      IF gpi.iEvent # NIL THEN
            (* This gadget is now active, change    *)
            (* visual state to selected and render. *)
        g := SYS.VAL (i.GadgetPtr, o);
        INCL (g.flags, i.gflgSelected);
        ignore := RenderRKMBut (cl, g, SYS.VAL (i.RenderPtr, msg));
        retval := i.gmrMeActive
      ELSE    (* The gmGoActive was not         *)
              (* triggered by direct user input *)
        retval := i.gmrNoReuse
      END;
    |
    i.gmRender :
      g := SYS.VAL (i.GadgetPtr, o);
      retval := RenderRKMBut (cl, g, SYS.VAL (i.RenderPtr, msg));
    |
    i.gmHandleInput : (* While it is active, this gadget sends its      *)
                      (* superclass an omNotify pulse for every         *)
                      (* classTimer event that goes by (about one every *)
                      (* 10th of a second).  Any object that is         *)
                      (* connected to this gadget will get A LOT of     *)
                      (* omUpdate messages.                             *)
      g := SYS.VAL (i.GadgetPtr, o);
      gpi := SYS.VAL (i.InputPtr, msg);
      ie := SYS.VAL (IE.InputEventPtr, gpi.iEvent);

      inst := bu.InstData (cl, o);

      retval := i.gmrMeActive;

      IF ORD (ie.class) = IE.classRawMouse THEN
        CASE ie.code OF
          i.selectUp :  (* The user let go of the gadget so return       *)
                        (* gmrNoReuse to deactivate and to tell          *)
                        (* Intuition not to reuse this Input Event as we *)
                        (* have already processed it.                    *)

                        (* If the user let go of the gadget while the    *)
                        (* mouse was over it, mask gmrVerify into the    *)
                        (* return value so Intuition will send a Release *)
                        (* Verify (gadgetUp).                            *)
            IF
              (gpi.mouse.x < g.leftEdge) OR
              (gpi.mouse.x > g.leftEdge + g.width) OR
              (gpi.mouse.y < g.topEdge) OR
              (gpi.mouse.y > g.topEdge + g.height)
            THEN
              retval := i.gmrNoReuse + i.gmrVerify
            ELSE
              retval := i.gmrNoReuse
            END;

                     (* Since the gadget is going inactive, send a final *)
                     (* notification to the icaTarget                    *)
            NotifyPulse (cl, o, {}, inst.midX, gpi)
          |
          i.menuDown :  (* The user hit the menu button. Go inactive and *)
                        (* let Intuition reuse the menu button event so  *)
                        (* Intuition can pop up the menu bar.            *)
            retval := i.gmrReuse;
            NotifyPulse (cl, o, {}, inst.midX, gpi)
          |
        ELSE
          retval := i.gmrMeActive
        END
      ELSIF ORD (ie.class) = IE.classTimer THEN
            (* If the gadget gets a timer event, it sends an interim *)
            (* omNotify to its superclass.                           *)
        NotifyPulse (cl, o, {i.opuInterim}, inst.midX, gpi)
      END;
    |
    i.gmGoInactive :      (* Intuition said to go inactive.  Clear the    *)
                          (* gflgSelected bit and render using unselected *)
                          (* imagery.                                     *)
      g := SYS.VAL (i.GadgetPtr, o);
      EXCL (g.flags, i.gflgSelected);
      ignore := RenderRKMBut (cl, g, SYS.VAL (i.RenderPtr, msg));
    |
    i.omSet :    (* Although this class doesn't have settable attributes, *)
                 (* this gadget class does have scaleable imagery, so it  *)
                 (* needs to find out when its size and/or position has   *)
                 (* changed so it can erase itself, THEN scale, and       *)
                 (* rerender.                                             *)
      opSet := SYS.VAL (i.OpSetPtr, msg);
      IF
        (u.base.FindTagItem (i.gaWidth, opSet.attrList) # NIL) OR
        (u.base.FindTagItem (i.gaHeight, opSet.attrList) # NIL) OR
        (u.base.FindTagItem (i.gaTop, opSet.attrList) # NIL) OR
        (u.base.FindTagItem (i.gaLeft, opSet.attrList) # NIL)
      THEN
        g := SYS.VAL (i.GadgetPtr, o);

        x := g.leftEdge;
        y := g.topEdge;
        wd := g.width;
        h := g.height;

        inst := bu.InstData (cl, o);

        retval := bu.DoSuperMethodA (cl, o, msg^);

                                    (* Get pointer to RastPort for gadget *)
        rp := i.base.ObtainGIRPort (opSet.gInfo);
        IF rp # NIL THEN
          pens := opSet.gInfo.drInfo.pens;
          gfx.base.SetAPen (rp, SHORT (pens [i.backGroundPen]));
          gfx.base.SetDrMd (rp, gfx.jam1);       (* Erase the old gadget. *)
          gfx.base.RectFill (rp, x, y, x+wd, y+h);

          inst.midX := g.leftEdge + (g.width DIV 2); (* Recalculate where *)
          inst.midY := g.topEdge + (g.height DIV 2); (* the center of the *)
                                                     (* gadget is. *)

                                                  (* Rerender the gadget. *)
          ignore :=
            bu.DoMethod (o, i.gmRender, opSet.gInfo, rp, i.gRedrawRedraw);
          i.base.ReleaseGIRPort (rp)
        END;
      ELSE
        retval := bu.DoSuperMethodA (cl, o, msg^)
      END;
    |
  ELSE (* rkmbutgadclass does not recognize the methodID, let the *)
       (* superclass's dispatcher take a look at it. *)
    retval := bu.DoSuperMethodA (cl, o, msg^);
  END;
  (* SYS.INLINE (04CDFH, 04CFCH); *)
  RETURN SYS.VAL (e.APTR, retval)
END dispatchRKMButGad;

(*------------------------------------*)
PROCEDURE initRKMButGadClass () : i.IClassPtr;

  VAR
    cl : i.IClassPtr;

BEGIN (* initRKMButGadClass *)
  cl := i.base.MakeClass ( "", "gadgetclass", NIL, SIZE (ButINST), {} );
  IF cl # NIL THEN
    (* initialize the IClass Hook *)
    cl.entry := hu.HookEntry;
    cl.subEntry := dispatchRKMButGad;
  END;
  RETURN cl
END initRKMButGadClass;


(*------------------------------------*)
PROCEDURE freeRKMButGadClass ( cl : i.IClassPtr );

  VAR ignore : BOOLEAN;

BEGIN (* freeRKMButGadClass *)
  ignore := i.base.FreeClass (cl)
END freeRKMButGadClass;

(*------------------------------------*)
PROCEDURE Main ();

  VAR ignore : INTEGER;

BEGIN (* Main *)
  IF i.base.version >= 37 THEN
    IF u.base.version >= 37 THEN
      IF gfx.base.version >= 37 THEN
        w := i.base.OpenWindowTagsA
          ( NIL,
            i.waFlags,  { i.wflgDepthGadget, i.wflgDragBar,
                          i.wflgCloseGadget, i.wflgSizeGadget },
            i.waIDCMP,  {i.idcmpCloseWindow},
            i.waWidth,  640,
            i.waHeight, 200,
            u.tagEnd );
        IF w # NIL THEN
          IF i.base.WindowLimits (w, 450, 200, 640, 200) THEN END;
          rkmbutcl := initRKMButGadClass();
          IF rkmbutcl # NIL THEN
            integer := i.base.NewObject
              ( NIL, "strgclass",
                i.gaID,            1,
                i.gaTop,           LONG (w.borderTop) + 5,
                i.gaLeft,          LONG (w.borderLeft) + 5,
                i.gaWidth,         intWidth,
                i.gaHeight,        intHeight,
                i.stringaLongVal,  0,
                i.stringaMaxChars, 5,
                u.tagEnd );
            IF integer # NIL THEN
              but := i.base.NewObject
                ( rkmbutcl, "",
                  i.gaID,            2,
                  i.gaTop,           LONG (w.borderTop) + 5,
                  i.gaLeft,          integer.leftEdge + integer.width + 5,
                  i.gaWidth,         40,
                  i.gaHeight,        intHeight,
                  i.gaPrevious,      integer,
                  i.icaMap,          SYS.ADR (pulse2int),
                  i.icaTarget,       integer,
                  u.tagEnd );
              IF but # NIL THEN
                ignore := i.base.AddGList (w, integer, -1, -1, NIL);
                i.base.RefreshGList (integer, w, NIL, -1);

                i.base.SetWindowTitles
                  ( w, SYS.ADR ("<-- Click to resize gadget Height"), NIL );
                MainLoop (u.tagDone, 0);

                i.base.SetWindowTitles
                  ( w, SYS.ADR ("<-- Click to resize gadget Width"), NIL );
                MainLoop (i.gaHeight, 100);

                i.base.SetWindowTitles
                  ( w, SYS.ADR ("<-- Click to resize gadget Y position"), NIL );
                MainLoop (i.gaWidth, 100);

                i.base.SetWindowTitles
                  ( w, SYS.ADR ("<-- Click to resize gadget X position"), NIL );
                MainLoop (i.gaTop, but.topEdge + 20);

                i.base.SetWindowTitles
                  ( w, SYS.ADR ("<-- Click to quit"), NIL );
                MainLoop (i.gaLeft, but.leftEdge + 20);

                ignore := i.base.RemoveGList (w, integer, -1);
                i.base.DisposeObject (but); but := NIL
              END;
              i.base.DisposeObject (integer); integer := NIL
            END;
            freeRKMButGadClass (rkmbutcl); rkmbutcl := NIL
          END;
          i.base.CloseWindow (w); w := NIL
        END;
      END;
    END;
  END
END Main;

BEGIN (* RKMButtonclass *)
  Init ();
  Main ();
END RKMButtonclass.

(*************************************************************************

  $Log: RKMButtonclass.mod $
# Revision 1.1  1994/08/08  16:55:25  fjc
# Release 1.4
#
*************************************************************************)

