(*************************************************************************

     $RCSfile: Talk2boopsi.mod $
  Description: A port of Talk2boopsi.c

               "This example creates a Boopsi prop gadget and integer
               string gadget, connecting them so they update each other
               when the user changes their value.  The example program
               only initializes the gadgets and puts them on the window;
               it doesn't have to interact with them to make them talk
               to each other."

   Created by: fjc (Frank Copeland)
    $Revision: 1.3 $
      $Author: fjc $
        $Date: 1994/08/08 16:55:25 $

  Copyright  1994, Frank Copeland.
  This example program is part of Oberon-A.
  See Oberon-A.doc for conditions of use and distribution.

  Log entries are at the end of the file.

*************************************************************************)

MODULE Talk2boopsi;

(*
** $C= CaseChk       $I= IndexChk  $L= LongAdr   $N= NilChk
** $P- PortableCode  $R= RangeChk  $S= StackChk  $T= TypeChk
** $V= OvflChk       $Z= ZeroVars
*)

IMPORT SYS := SYSTEM, E := Exec, U := Utility, I := Intuition;

CONST
  VersionTag = "\0$VER: Talk2boopsi 1.0 (9.6.94)\r\n";

VAR
  w : I.WindowPtr;
  msg : I.IntuiMessagePtr;
  prop, integer : I.GadgetPtr;
  prop2intmap, int2propmap : ARRAY 2 OF U.TagItem;

CONST
  propGadgetID = 1;
  intGadgetID = 2;
  propGadgetWidth = 10;
  propGadgetHeight = 80;
  intGadgetHeight = 18;
  visible = 10;
  total = 100;
  initialVal = 25;
  minWindowWidth = 80;
  minWindowHeight = propGadgetHeight + 70;
  maxChars = 3;

(*------------------------------------*)
PROCEDURE* Cleanup ();

BEGIN (* Cleanup *)
  IF integer # NIL THEN
    SYS.PUTREG (0, I.base.RemoveGList (w, prop, -1));
    I.base.DisposeObject (integer);
  END;
  IF prop # NIL THEN
    I.base.DisposeObject (prop);
  END;
  IF w # NIL THEN
    I.base.CloseWindow (w);
  END;
END Cleanup;

(*------------------------------------*)
PROCEDURE Init ();

BEGIN (* Init *)
  prop2intmap [0].tag := I.pgaTop;
  prop2intmap [0].data := I.stringaLongVal;
  prop2intmap [1].tag := U.tagEnd;
  int2propmap [0].tag := I.stringaLongVal;
  int2propmap [0].data := I.pgaTop;
  int2propmap [1].tag := U.tagEnd;
  w := NIL; prop := NIL; integer := NIL;

  SYS.SETCLEANUP (Cleanup);
END Init;

(*------------------------------------*)
PROCEDURE Main ();

  VAR done : BOOLEAN;

BEGIN (* Main *)
  done := FALSE;
  (*
    Open the window -- notice that the window's IDCMP port does not
    listen for GADGETUP messages.
  *)
  w := I.base.OpenWindowTagsA (
      NIL,
      I.waFlags,     { I.wflgDepthGadget, I.wflgDragBar,
                       I.wflgCloseGadget, I.wflgSizeGadget },
      I.waIDCMP,     { I.idcmpCloseWindow },
      I.waMinWidth,  minWindowWidth,
      I.waMinHeight, minWindowHeight,
      U.tagEnd );
  IF w # NIL THEN
    (* Create a new propgclass object *)
    prop := I.base.NewObject ( NIL, "propgclass",
        I.gaID,       propGadgetID,
        I.gaTop,      w.borderTop + 5,
        I.gaLeft,     w.borderLeft + 5,
        I.gaWidth,    propGadgetWidth,
        I.gaHeight,   propGadgetHeight,
        I.icaMap,     SYS.ADR (prop2intmap),
        I.pgaTotal,   total,
        I.pgaTop,     initialVal,
        I.pgaVisible, visible,
        I.pgaNewLook, TRUE,
        U.tagEnd );
    IF prop # NIL THEN
      (* Create the integer string gadget *)
      integer := I.base.NewObject ( NIL, "strgclass",
          I.gaID,            intGadgetID,
          I.gaTop,           w.borderTop + 5,
          I.gaLeft,          w.borderLeft + propGadgetWidth + 5,
          I.gaWidth,         minWindowWidth -
                             ( w.borderLeft + w.borderRight +
                               propGadgetWidth + 15 ),
          I.gaHeight,        intGadgetHeight,
          I.icaMap,          SYS.ADR (int2propmap),
          I.icaTarget,       prop,
          I.gaPrevious,      prop,
          I.stringaLongVal,  initialVal,
          I.stringaMaxChars, maxChars,
          U.tagEnd );
      IF integer # NIL THEN
        SYS.PUTREG (0, I.base.SetGadgetAttrs (
            prop^, w, NIL,
            I.icaTarget, integer,
            U.tagEnd ));
        SYS.PUTREG (0, I.base.AddGList (w, prop, -1, -1, NIL));
        I.base.RefreshGList (prop, w, NIL, -1);
        WHILE ~done DO
          E.base.WaitPort (w.userPort);
          LOOP
            msg :=
              SYS.VAL (I.IntuiMessagePtr, E.base.GetMsg (w.userPort));
            IF msg = NIL THEN EXIT END;
            IF msg.class = {I.idcmpCloseWindow} THEN done := TRUE END;
            E.base.ReplyMsg (msg);
          END
        END
      END
    END
  END
END Main;

BEGIN (* Talk2boopsi *)
  Init ();
  Main ();
END Talk2boopsi.

(*************************************************************************

  $Log: Talk2boopsi.mod $
  Revision 1.3  1994/08/08  16:55:25  fjc
  Release 1.4

  Revision 1.2  1994/06/14  01:00:59  fjc
  - Updated for release

  Revision 1.1  1994/06/09  22:09:57  oberon
  - Initial revision

*************************************************************************)

