(*************************************************************************

     $RCSfile: AppWindow.mod $
  Description:

   Created by: fjc (Frank Copeland)
    $Revision: 1.2 $
      $Author: fjc $
        $Date: 1994/08/08 16:59:51 $

  Copyright  1994, Frank Copeland.
  This example program is part of Oberon-A.
  See Oberon-A.doc for conditions of use and distribution.

  Log entries are at the end of the file.

*************************************************************************)

MODULE AppWindow;

(*
** $C= CaseChk       $I= IndexChk  $L= LongAdr   $N= NilChk
** $P- PortableCode  $R= RangeChk  $S= StackChk  $T= TypeChk
** $V= OvflChk       $Z= ZeroVars
*)

IMPORT
  SYS := SYSTEM,
  e   := Exec,
  u   := Utility,
  I   := Intuition,
  wb  := Workbench,
  IO  := StdIO;

CONST
  VersionTag = "\0$VER: AppWindow 1.0 (20.6.94)";

VAR

  awport : e.MsgPortPtr;
  win    : I.WindowPtr;
  appwin : wb.AppWindowPtr;
  imsg   : I.IntuiMessagePtr;
  amsg   : wb.AppMessagePtr;

  winsig, appwinsig : SHORTINT;
  id, userdata : LONGINT;
  signals : SET;
  done : BOOLEAN;
  i : INTEGER;

BEGIN (* AppWindow *)
  IF I.base.version >= 37 THEN
    wb.OpenLib (FALSE);
    IF wb.base.version >= 37 THEN
      awport := e.base.CreateMsgPort ();
      IF awport # NIL THEN
        win := I.base.OpenWindowTagsA
          ( NIL,
            I.waWidth,  200,
            I.waHeight, 50,
            I.waIDCMP,  {I.idcmpCloseWindow},
            I.waFlags,  {I.wflgCloseGadget, I.wflgDragBar},
            I.waTitle,  SYS.ADR ("AppWindow"),
            u.tagDone );
        IF win # NIL THEN
          id := 1; userdata := 0;
          appwin := wb.base.AddAppWindow
            ( id, userdata, win, awport, NIL );
          IF appwin # NIL THEN
            IO.WriteStr ("AppWindow added... Drag files into AppWindow\n");
            winsig := win.userPort.sigBit;
            appwinsig := awport.sigBit;
            done := FALSE;
            WHILE ~done DO
              (* Wait for IDCMP messages and AppMessages *)
              signals := e.base.Wait ({winsig, appwinsig});
              IF winsig IN signals THEN (* Got an IDCMP message *)
                LOOP
                  imsg := SYS.VAL (I.IntuiMessagePtr,
                    e.base.GetMsg (win.userPort));
                  IF imsg = NIL THEN EXIT END;
                  IF imsg.class = {I.idcmpCloseWindow} THEN
                    done := TRUE
                  END;
                  e.base.ReplyMsg (imsg)
                END;
              END;
              IF appwinsig IN signals THEN (* Got an AppMessage *)
                LOOP
                  amsg := SYS.VAL (wb.AppMessagePtr,
                    e.base.GetMsg (awport));
                  IF amsg = NIL THEN EXIT END;
                  IO.WriteF3
                    ( "AppMsg: Type=%ld, ID=%ld, NumArgs=%ld\n",
                      LONG (amsg.amType), amsg.id, amsg.numArgs );
                  FOR i := 0 TO amsg.numArgs - 1 DO
                    IO.WriteF3
                      ( "  arg(%ld): Name='%s', Lock=%lx\n",
                        LONG (i), amsg.argList[i].name, amsg.argList[i].lock )
                  END;
                  e.base.ReplyMsg (amsg)
                END;
              END;
            END; (* WHILE *)
            IF wb.base.RemoveAppWindow (appwin) THEN END
          END; (* IF *)
          I.base.CloseWindow (win)
        END; (* IF *)
        (* Make sure there are no more outstanding messages *)
        LOOP
          amsg := SYS.VAL (wb.AppMessagePtr,
            e.base.GetMsg (awport));
          IF imsg = NIL THEN EXIT END;
          e.base.ReplyMsg (amsg)
        END;
        e.base.DeleteMsgPort (awport)
      END; (* IF *)
    END; (* IF *)
  END; (* IF *)
END AppWindow.

(*************************************************************************

  $Log: AppWindow.mod $
  Revision 1.2  1994/08/08  16:59:51  fjc
  Release 1.4

  Revision 1.1  1994/07/03  15:17:26  fjc
  - Incorporated changes in 3.1 Interfaces

*************************************************************************)

