(**************************************************************************

     $RCSfile: Printer.mod $
  Description: Interface to printer.device

   Created by: fjc (Frank Copeland)
    $Revision: 3.2 $
      $Author: fjc $
        $Date: 1994/08/08 01:10:41 $

  Includes Release 40.15

  (C) Copyright 1987-1993 Commodore-Amiga, Inc.
      All Rights Reserved

  Oberon-A interface Copyright  1994, Frank Copeland.
  This file is part of the Oberon-A Interface.
  See Oberon-A.doc for conditions of use and distribution.

***************************************************************************)

MODULE Printer;

(*
** $C- CaseChk       $I- IndexChk  $L+ LongAdr   $N- NilChk
** $P- PortableCode  $R- RangeChk  $S- StackChk  $T- TypeChk
** $V- OvflChk       $Z- ZeroVars
*)

IMPORT
  E := Exec, TI := Timer, P := Parallel, S := Serial, G := Graphics,
  I := Intuition, SYS := SYSTEM;


(*
**      $VER: printer.h 1.7 (26.7.90)
**
**      printer.device structure definitions
*)

CONST

  rawWrite *   =   E.cmdNonstd+0;
  prtCommand * =   E.cmdNonstd+1;
  dumpRPort *  =   E.cmdNonstd+2;
  query *      =   E.cmdNonstd+3;

(* printer command definitions *)

  aRIS *    = 0;   (* ESCc  reset                    ISO *)
  aRIN *    = 1;   (* ESC#1 initialize       +++ *)
  aIND *    = 2;   (* ESCD  lf               ISO *)
  aNEL *    = 3;   (* ESCE  return,lf                ISO *)
  aRI *     = 4;   (* ESCM  reverse lf       ISO *)

  aSGR0 *   = 5;   (* ESC[0m normal char set         ISO *)
  aSGR3 *   = 6;   (* ESC[3m italics on              ISO *)
  aSGR23 *  = 7;   (* ESC[23m italics off            ISO *)
  aSGR4 *   = 8;   (* ESC[4m underline on            ISO *)
  aSGR24 *  = 9;   (* ESC[24m underline off          ISO *)
  aSGR1 *   = 10;  (* ESC[1m boldface on             ISO *)
  aSGR22 *  = 11;  (* ESC[22m boldface off           ISO *)
  aSFC *    = 12;  (* SGR30-39  set foreground color ISO *)
  aSBC *    = 13;  (* SGR40-49  set background color ISO *)

  aSHORP0 * = 14;  (* ESC[0w normal pitch            DEC *)
  aSHORP2 * = 15;  (* ESC[2w elite on                DEC *)
  aSHORP1 * = 16;  (* ESC[1w elite off       DEC *)
  aSHORP4 * = 17;  (* ESC[4w condensed fine on       DEC *)
  aSHORP3 * = 18;  (* ESC[3w condensed off           DEC *)
  aSHORP6 * = 19;  (* ESC[6w enlarged on             DEC *)
  aSHORP5 * = 20;  (* ESC[5w enlarged off            DEC *)

  aDEN6 *   = 21;  (* ESC[6"z shadow print on        DEC (sort of) *)
  aDEN5 *   = 22;  (* ESC[5"z shadow print off       DEC *)
  aDEN4 *   = 23;  (* ESC[4"z doublestrike on        DEC *)
  aDEN3 *   = 24;  (* ESC[3"z doublestrike off       DEC *)
  aDEN2 *   = 25;  (* ESC[2"z  NLQ on                DEC *)
  aDEN1 *   = 26;  (* ESC[1"z  NLQ off       DEC *)

  aSUS2 *   = 27;  (* ESC[2v superscript on          +++ *)
  aSUS1 *   = 28;  (* ESC[1v superscript off         +++ *)
  aSUS4 *   = 29;  (* ESC[4v subscript on            +++ *)
  aSUS3 *   = 30;  (* ESC[3v subscript off           +++ *)
  aSUS0 *   = 31;  (* ESC[0v normalize the line      +++ *)
  aPLU *    = 32;  (* ESCL  partial line up          ISO *)
  aPLD *    = 33;  (* ESCK  partial line down        ISO *)

  aFNT0 *   = 34;  (* ESC(B US char set        or Typeface  0 (default) *)
  aFNT1 *   = 35;  (* ESC(R French char set    or Typeface  1 *)
  aFNT2 *   = 36;  (* ESC(K German char set    or Typeface  2 *)
  aFNT3 *   = 37;  (* ESC(A UK char set        or Typeface  3 *)
  aFNT4 *   = 38;  (* ESC(E Danish I char set  or Typeface  4 *)
  aFNT5 *   = 39;  (* ESC(H Sweden char set    or Typeface  5 *)
  aFNT6 *   = 40;  (* ESC(Y Italian char set   or Typeface  6 *)
  aFNT7 *   = 41;  (* ESC(Z Spanish char set   or Typeface  7 *)
  aFNT8 *   = 42;  (* ESC(J Japanese char set  or Typeface  8 *)
  aFNT9 *   = 43;  (* ESC(6 Norweign char set  or Typeface  9 *)
  aFNT10 *  = 44;  (* ESC(C Danish II char set or Typeface 10 *)

(*
        Suggested typefaces are:

         0 - default typeface.
         1 - Line Printer or equiv.
         2 - Pica or equiv.
         3 - Elite or equiv.
         4 - Helvetica or equiv.
         5 - Times Roman or equiv.
         6 - Gothic or equiv.
         7 - Script or equiv.
         8 - Prestige or equiv.
         9 - Caslon or equiv.
        10 - Orator or equiv.
*)

  aPROP2 *  = 45;  (* ESC[2p  proportional on        +++ *)
  aPROP1 *  = 46;  (* ESC[1p  proportional off       +++ *)
  aPROP0 *  = 47;  (* ESC[0p  proportional clear     +++ *)
  aTSS *    = 48;  (* ESC[n E set proportional offset ISO *)
  aJFY5 *   = 49;  (* ESC[5 F auto left justify      ISO *)
  aJFY7 *   = 50;  (* ESC[7 F auto right justify     ISO *)
  aJFY6 *   = 51;  (* ESC[6 F auto full justify      ISO *)
  aJFY0 *   = 52;  (* ESC[0 F auto justify off       ISO *)
  aJFY3 *   = 53;  (* ESC[3 F letter space (justify) ISO (special) *)
  aJFY1 *   = 54;  (* ESC[1 F word fill(auto center) ISO (special) *)

  aVERP0 *  = 55;  (* ESC[0z  1/8" line spacing      +++ *)
  aVERP1 *  = 56;  (* ESC[1z  1/6" line spacing      +++ *)
  aSLPP *   = 57;  (* ESC[nt  set form length n      DEC *)
  aPERF *   = 58;  (* ESC[nq  perf skip n (n>0)      +++ *)
  aPERF0 *  = 59;  (* ESC[0q  perf skip off          +++ *)

  aLMS *    = 60;  (* ESC#9   Left margin set        +++ *)
  aRMS *    = 61;  (* ESC#0   Right margin set       +++ *)
  aTMS *    = 62;  (* ESC#8   Top margin set         +++ *)
  aBMS *    = 63;  (* ESC#2   Bottom marg set        +++ *)
  aSTBM *   = 64;  (* ESC[Pn1;Pn2r  T&B margins      DEC *)
  aSLRM *   = 65;  (* ESC[Pn1;Pn2s  L&R margin       DEC *)
  aCAM *    = 66;  (* ESC#3   Clear margins          +++ *)

  aHTS *    = 67;  (* ESCH    Set horiz tab          ISO *)
  aVTS *    = 68;  (* ESCJ    Set vertical tabs      ISO *)
  aTBC0 *   = 69;  (* ESC[0g  Clr horiz tab          ISO *)
  aTBC3 *   = 70;  (* ESC[3g  Clear all h tab        ISO *)
  aTBC1 *   = 71;  (* ESC[1g  Clr vertical tabs      ISO *)
  aTBC4 *   = 72;  (* ESC[4g  Clr all v tabs         ISO *)
  aTBCALL * = 73;  (* ESC#4   Clr all h & v tabs     +++ *)
  aTBSALL * = 74;  (* ESC#5   Set default tabs       +++ *)
  aEXTEND * = 75;  (* ESC[Pn"x extended commands     +++ *)

  aRAW *    = 76;      (* ESC[Pn"r     Next 'Pn' chars are raw +++ *)

TYPE

  IOPrtCmdReqPtr * = CPOINTER TO IOPrtCmdReq;
  IOPrtCmdReq * = RECORD (E.Message)
    device *     : E.DevicePtr; (* device node pointer  *)
    unit *       : E.UnitPtr;   (* unit (driver private)*)
    command *    : E.UWORD;     (* device command *)
    ioFlags *    : E.BSET;
    error *      : SHORTINT;    (* error or warning num *)
    prtCommand * : E.UWORD;     (* printer command *)
    parm0 *      : E.UBYTE;     (* first command parameter *)
    parm1 *      : E.UBYTE;     (* second command parameter *)
    parm2 *      : E.UBYTE;     (* third command parameter *)
    parm3 *      : E.UBYTE;     (* fourth command parameter *)
  END; (* IOPrtCmdReq *)

  IODRPReqPtr * = CPOINTER TO IODRPReq;
  IODRPReq * = RECORD (E.Message)
    device *    : E.DevicePtr;   (* device node pointer  *)
    unit *      : E.UnitPtr;     (* unit (driver private)*)
    command *   : E.UWORD;       (* device command *)
    flags *   : E.BSET;
    error *     : SHORTINT;      (* error or warning num *)
    rastPort *  : G.RastPortPtr; (* raster port *)
    colorMap *  : G.ColorMapPtr; (* color map *)
    modes *     : SET;           (* graphics viewport modes *)
    srcX *      : E.UWORD;       (* source x origin *)
    srcY *      : E.UWORD;       (* source y origin *)
    srcWidth *  : E.UWORD;       (* source x width *)
    srcHeight * : E.UWORD;       (* source x height *)
    destCols *  : LONGINT;       (* destination x width *)
    destRows *  : LONGINT;       (* destination y height *)
    special *   : E.WSET;        (* option flags *)
  END; (* IODRPReq *)

CONST

  specialMilCols *         = 0;  (* DestCols specified in 1/1000" *)
  specialMilRows *         = 1;  (* DestRows specified in 1/1000" *)
  specialFullCols *        = 2;  (* make DestCols maximum possible *)
  specialFullRows *        = 3;  (* make DestRows maximum possible *)
  specialFracCols *        = 4;  (* DestCols is fraction of FULLCOLS *)
  specialFracRows *        = 5;  (* DestRows is fraction of FULLROWS *)
  specialCenter *          = 6;  (* center image on paper *)
  specialAspect *          = 7;  (* ensure correct aspect ratio *)
  specialDensity1 *        = {8};  (* lowest resolution (dpi) *)
  specialDensity2 *        = {9};  (* next res *)
  specialDensity3 *        = {8,9};  (* next res *)
  specialDensity4 *        = {10};  (* next res *)
  specialDensity5 *        = {8,10};  (* next res *)
  specialDensity6 *        = {9,10};  (* next res *)
  specialDensity7 *        = {8,9,10};  (* highest res *)
  specialNoFormFeed *      = 11;  (* don't eject paper on gfx prints *)
  specialTrustMe *         = 12;  (* don't reset on gfx prints *)
(*
        Compute print size, set 'ioDestCols' and 'ioDestRows' in the calling
        program's 'IODRPReq' structure and exit, DON'T PRINE.  This allows the
        calling program to see what the final print size would be in printer
        pixels.  Note that it modifies the 'ioDestCols' and 'ioDestRows'
        fields of your 'IODRPReq' structure.  Also, set the print density and
        update the 'MaxXDots', 'MaxYDots', 'XDotsInch', and 'YDotsInch' fields
        of the 'PrinterExtendedData' structure.
*)
  specialNoPrint *         = 13;  (* see above *)

  errNoErr *             = 0;       (* clean exit, no errors *)
  errCancel *            = 1;       (* user cancelled print *)
  errNotGraphics *       = 2;       (* printer cannot output graphics *)
  errInvertHam *         = 3;       (* OBSOLETE *)
  errBadDimension *      = 4;       (* print dimensions illegal *)
  errDimensionOvflow *   = 5;       (* OBSOLETE *)
  errInternalMemory *    = 6;       (* no memory for internal variables *)
  errBufferMemory *      = 7;       (* no memory for print buffer *)
(*
        Note * : this is an internal error that can be returned from the render
        function to the printer device.  It is NEVER returned to the user.
        If the printer device sees this error it converts it 'PderrNOERR'
        and exits gracefully.  Refer to the document on
        'How to Write a Graphics Printer Driver' for more info.
*)
  errTookControl *       = 8;       (* Took control in case 0 of render *)

(* internal use *)
  specialDensityMask *     = {8..10};  (* masks out density values *)
  specialDimensionsMask *  =
    { specialMilCols .. specialFracRows, specialAspect };

(*
**      $VER: prtgfx.h 1.12 (26.7.90)
**
**      printer.device structure definitions
*)

CONST

  pcmYellow *       = 0;               (* byte index for yellow *)
  pcmMagenta *      = 1;               (* byte index for magenta *)
  pcmCyan *         = 2;               (* byte index for cyan *)
  pcmBlack *        = 3;               (* byte index for black *)
  pcmBlue *         = pcmYellow;       (* byte index for blue *)
  pcmGreen *        = pcmMagenta;      (* byte index for green *)
  pcmRed *          = pcmCyan;         (* byte index for red *)
  pcmWhite *        = pcmBlack;        (* byte index for white *)

TYPE

  colorEntryPtr * = CPOINTER TO colorEntry;
  colorEntry * = RECORD
  (*colorLong *  : E.ULONG;            (* quick access to all of YMCB *)*)
    colorByte *  : ARRAY 4 OF E.UBYTE; (* 1 entry for each of YMCB *)
  (*colorSByte * : SHORTINT[4];        (* ditto (except signed) *)*)
  END; (* colorEntry *)

  PrtInfoPtr * = CPOINTER TO PrtInfo;
  PrtInfo * = RECORD
    render        : PROCEDURE (): LONGINT; (* PRIVATE - DO NOT USE! *)
    rp            : G.RastPortPtr; (* PRIVATE - DO NOT USE! *)
    temprp        : G.RastPortPtr; (* PRIVATE - DO NOT USE! *)
    rowBuf        : E.APTR;        (* PRIVATE - DO NOT USE! *)
    hamBuf        : E.APTR;        (* PRIVATE - DO NOT USE! *)
    colorMap      : colorEntryPtr; (* PRIVATE - DO NOT USE! *)
    colorInt *    : colorEntryPtr; (* color intensities for entire row *)
    hamInt        : colorEntryPtr; (* PRIVATE - DO NOT USE! *)
    dest1Int      : colorEntryPtr; (* PRIVATE - DO NOT USE! *)
    dest2Int      : colorEntryPtr; (* PRIVATE - DO NOT USE! *)
    scaleX *      : E.APTR;        (* array of scale values for X *)
    scaleXAlt     : E.APTR;        (* PRIVATE - DO NOT USE! *)
    dmatrix *     : E.APTR;        (* pointer to dither matrix *)
    topBuf        : E.APTR;        (* PRIVATE - DO NOT USE! *)
    botBuf        : E.APTR;        (* PRIVATE - DO NOT USE! *)

    rowBufSize    : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    hamBufSize    : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    colorMapSize  : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    colorIntSize  : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    hamIntSize    : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    dest1IntSize  : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    dest2IntSize  : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    scaleXSize    : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    scaleXAltSize : E.UWORD;       (* PRIVATE - DO NOT USE! *)

    prefsFlags    : E.WSET;        (* PRIVATE - DO NOT USE! *)
    special       : E.ULONG;       (* PRIVATE - DO NOT USE! *)
    xstart        : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    ystart        : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    width *       : E.UWORD;       (* source width (in pixels) *)
    height        : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    pc            : E.ULONG;       (* PRIVATE - DO NOT USE! *)
    pr            : E.ULONG;       (* PRIVATE - DO NOT USE! *)
    ymult         : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    ymod          : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    ety           : INTEGER;       (* PRIVATE - DO NOT USE! *)
    xpos *        : E.UWORD;       (* offset to start printing picture *)
    threshold *   : E.UWORD;       (* threshold value (from prefs) *)
    tempwidth     : E.UWORD;       (* PRIVATE - DO NOT USE! *)
    flags         : E.WSET;        (* PRIVATE - DO NOT USE! *)
  END; (* PrtInfo *)

(*
**      $VER: prtbase.h 1.10 (2.11.90)
**
**      printer.device base structure definitions
*)

TYPE

  DeviceDataPtr * = CPOINTER TO DeviceData;
  DeviceData * = RECORD (E.Library) (* standard library node *)
    segment *     : SYS.BPTR;       (* A0 when initialized *)
    execBase *    : E.LibraryPtr;   (* A6 for exec *)
    cmdVectors *  : E.APTR;         (* command table for device commands *)
    cmdBytes *    : E.APTR;         (* bytes describing which command queue *)
    numCommands * : E.UWORD;        (* the number of commands supported *)
  END; (* DeviceData *)

CONST

  oldStkSize *    = 0800H;  (* stack size for child task (OBSOLETE) *)
  stkSize *       = 1000H;  (* stack size for child task *)
  bufSize *       = 256;    (* size of internal buffers for text i/o *)
  safeSize *      = 128;    (* safety margin for text output buffer *)

TYPE

  PrinterExtendedDataPtr * = CPOINTER TO PrinterExtendedData;
  PrinterExtendedData * = RECORD
    printerName *  : E.STRPTR;  (* printer name, null terminated *)
    init *         : PROCEDURE; (* called after LoadSeg *)
    expunge *      : PROCEDURE; (* called before UnLoadSeg *)
    open *         : PROCEDURE () : LONGINT; (* called at OpenDevice *)
    close *        : PROCEDURE; (* called at CloseDevice *)
    printerClass * : E.BSET;    (* printer class *)
    colorClass *   : E.BSET;    (* color class *)
    maxColumns *   : E.UBYTE;   (* number of print columns available *)
    numCharSets *  : E.UBYTE;   (* number of character sets *)
    numRows *      : E.UWORD;   (* number of 'pins' in print head *)
    maxXDots *     : E.ULONG;   (* number of dots max in a raster dump *)
    maxYDots *     : E.ULONG;   (* number of dots max in a raster dump *)
    xDotsInch *    : E.UWORD;   (* horizontal dot density *)
    yDotsInch *    : E.UWORD;   (* vertical dot density *)
    commands *     : E.APTR;    (* printer text command table *)
    doSpecial *    : PROCEDURE () : LONGINT; (* Special command handler *)
    render *       : PROCEDURE () : LONGINT; (* raster render function *)
    timeoutSecs *  : LONGINT;   (* good write timeout *)
    (* the following only exists if the segment version is >= 33 *)
    eight8BitChars * : E.APTR;    (* conv. strings for the extended font *)
    printMode *    : LONGINT;   (* set if text printed, otherwise 0 *)
    (* the following only exists if the segment version is >= 34 *)
    (* ptr to conversion function for all chars *)
    convFunc *     : PROCEDURE () : LONGINT;
  END; (* PrinterExtendedData *)

  PrinterSegmentPtr * = CPOINTER TO PrinterSegment;
  PrinterSegment * = RECORD
    nextSegment * : SYS.BPTR; (* (actually a BPTR) *)
    runAlert *    : E.ULONG;  (* MOVEQ #0,D0 * : RTS *)
    version *     : E.UWORD;  (* segment version *)
    revision *    : E.UWORD;  (* segment revision *)
    ped *         : PrinterExtendedData;   (* printer extended data *)
  END; (* PrinterSegment *)

  PrinterDataPtr * = CPOINTER TO PrinterData;
  PrinterData * = RECORD (DeviceData)
    unit *         : E.MsgPort;   (* the one and only unit *)
    printerSegment * : SYS.BPTR;  (* the printer specific segment *)
    printerType *  : E.UWORD;     (* the segment printer type *)
                                  (* the segment data structure *)
    segmentData *  : PrinterSegmentPtr;
    printBuf *     : E.APTR;    (* the raster print buffer *)
    pWrite *       : PROCEDURE () : LONGINT; (* the write function *)
    pBothReady *   : PROCEDURE () : LONGINT; (* write function's done *)
    p0 *           : P.IOExtPar; (* port I/O request 0 *)
    p0pad          : ARRAY (SIZE (S.IOExtSer) - SIZE (P.IOExtPar)) OF SYS.BYTE;
    p1 *           : P.IOExtPar; (*   and 1 for double buffering *)
    p1pad          : ARRAY (SIZE (S.IOExtSer) - SIZE (P.IOExtPar)) OF SYS.BYTE;
    tior *         : TI.TimeRequest; (* timer I/O request *)
    iorPort *      : E.MsgPort;      (* and message reply port *)
    tc *           : E.Task;         (* write task *)
    oldStk *       : ARRAY oldStkSize OF SYS.BYTE;
                                     (* and stack space (OBSOLETE) *)
    pdFlags *      : E.BSET;         (* device flags *)
    pdPad *        : E.UBYTE;        (* padding *)
    preferences *  : I.Preferences;  (* the latest preferences *)
    pWaitEnabled * : E.UBYTE;        (* wait function switch *)
    (* new fields for V2.0 *)
    flags1 *       : E.BSET;         (* padding *)
    stk *          : ARRAY stkSize OF SYS.BYTE;
                                     (* stack space *)
  END; (* PrinterData *)

CONST

(* Printer Class *)
  pcGfx *         = 0;      (* graphics (bit position) *)
  pcColor *       = 1;      (* color (bit position) *)

  pcBwAlpha *     = {};               (* black&white alphanumerics *)
  pcBwGfx *       = {pcGfx};          (* black&white graphics *)
  pcColorAlpha *  = {pcColor};        (* color alphanumerics *)
  pcColorGfx *    = {pcGfx, pcColor}; (* color graphics *)

(* Color Class *)
  ccBw *          = {0};       (* black&white only *)
  ccYmc *         = {1};       (* yellow/magenta/cyan only *)
  ccYmcBW *       = {0, 1};    (* yellow/magenta/cyan or black&white *)
  ccYmcb *        = 2;         (* yellow/magenta/cyan/black *)
  ccFourcolor *   = 2;         (* a flag for YMCB and BGRW *)
  ccAdditive *    = 3;         (* not ymcb but blue/green/red/white *)
  ccWb *          = {0, 3};    (* black&white only, 0 * == BLACK *)
  ccBgr *         = {1, 3};    (* blue/green/red *)
  ccBgrWB *       = {0, 1, 3}; (* blue/green/red or black&white *)
  ccBgrw *        = {2, 3};    (* blue/green/red/white *)
(*
        The picture must be scanned once for each color component, as the
        printer can only define one color at a time.  ie. If 'PccYMC' then
        first pass sends all 'Y' info to printer, second pass sends all 'M'
        info, and third pass sends all C info to printer.  The CalComp
        PlotMaster is an example of this type of printer.
*)
  ccMultiPass *  = 4;    (* see explanation above *)

CONST

  name * = "printer.device";

END Printer.
