(***************************************************************************

     $RCSfile: Fonts.mod $
  Description: Port of the Project Oberon Fonts module

   Created by: J. Gutknecht
    Ported by: fjc (Frank Copeland)
    $Revision: 1.7 $
      $Author: fjc $
        $Date: 1994/08/08 16:40:58 $

  Copyright  1990-1993, ETH Zuerich
  Copyright  1994, Frank Copeland.
  This file is part of the Oberon-A Library.
  See Oberon-A.doc for conditions of use and distribution.

  Log entries are at the end of the file.

***************************************************************************)

MODULE Fonts;

(*
** $C= CaseChk       $I= IndexChk  $L= LongAdr   $N= NilChk
** $P= PortableCode  $R= RangeChk  $S= StackChk  $T= TypeChk
** $V= OvflChk       $Z= ZeroVars
*)

IMPORT
  SYS := SYSTEM, Exec, Gfx := Graphics, DF := DiskFont,
  Str := Strings;

TYPE

  Name * = ARRAY 32 OF CHAR;

  Font * = POINTER TO FontDesc;
  FontDesc * = RECORD
    name * : Name;
    height * : INTEGER;
    textAttr * : Gfx.TextAttrPtr;
    textFont * : Gfx.TextFontPtr;
    next : Font;
  END; (* FontDesc *)

VAR

  Default *, First : Font;
  nofFonts : INTEGER;

(*------------------------------------*)
PROCEDURE This * (name : ARRAY OF CHAR) : Font;
(*
 *  Opens the font described by name and returns a descriptor for it.
 *  name is required to be in the common Amiga notation for fonts, namely:
 *
 *    <font name>/<font size>
 *
 *  For example: "topaz/8" refers to the size 8 version of the topaz font.
 *)

  VAR
    F : Font; fontName : Name; len, pos : LONGINT; size, i : INTEGER;
    ch : CHAR; textAttr : Gfx.TextAttrPtr; textFont : Gfx.TextFontPtr;

(* $D- disable copying of open arrays *)
BEGIN (* This *)
  F := First; WHILE (F # NIL) & (name # F.name) DO F := F.next END;
  IF F = NIL THEN
    COPY (name, fontName);
    pos := Str.FindChar ("/", fontName, 0);
    IF pos >= 0 THEN
      len := Str.Length (fontName);
      IF len > (pos + 1) THEN
        i := SHORT (pos) + 1; size := 0; ch := fontName [i];
        WHILE (i < len) & (ch >= "0") & (ch <= "9") DO
          size := (size * 10) + (ORD (ch) - ORD ("0"));
          INC (i); ch := fontName [i]
        END; (* WHILE *)
        IF i = len THEN
          fontName [pos] := 0X; Str.Append (fontName, ".font");
          NEW (textAttr);
          SYS.NEW (textAttr.name, Str.Length (fontName) + 1);
          COPY (fontName, textAttr.name^);
          textAttr.ySize := size;
          textAttr.style := Gfx.fsNormal;
          textAttr.flags := {Gfx.fpDiskFont};
          textFont := DF.base.OpenDiskFont (textAttr^);
          IF textFont # NIL THEN
            NEW (F);
            COPY (name, F.name); F.height := size;
            F.textAttr := textAttr; F.textFont := textFont;
            F.next := First; First := F
          ELSE
            SYS.DISPOSE (textAttr.name); SYS.DISPOSE (textAttr);
            F := Default
          END; (* ELSE *)
        ELSE
          F := Default
        END; (* ELSE *)
      ELSE
        F := Default
      END; (* ELSE *)
    ELSE
      F := Default
    END; (* ELSE *)
  END; (* IF *)
  RETURN F
END This;

(*------------------------------------*)
PROCEDURE* Cleanup ();

  VAR F : Font;

BEGIN (* Cleanup *)
  F := First;
  WHILE F # Default DO
    IF F.textFont # NIL THEN Gfx.base.CloseFont (F.textFont) END;
    F := F.next
  END;
END Cleanup;

(*------------------------------------*)
PROCEDURE GetDefault ();

  VAR df : Gfx.TextFontPtr; ta : Gfx.TextAttrPtr;

BEGIN (* GetDefault *)
  df := Gfx.base.defaultFont;
  NEW (ta);
  ta.name := df.name; ta.ySize := df.ySize;
  ta.style := df.style; ta.flags := df.flags;
  NEW (Default);
  COPY (df.name^, Default.name); Default.height := df.ySize;
  Default.textAttr := ta; Default.textFont := df; Default.next := NIL;
END GetDefault;

BEGIN (* Fonts *)
  DF.OpenLib (TRUE);
  GetDefault (); First := Default; nofFonts := 1;
  SYS.SETCLEANUP (Cleanup)
END Fonts.

(***************************************************************************

  $Log: Fonts.mod $
  Revision 1.7  1994/08/08  16:40:58  fjc
  Release 1.4

  Revision 1.6  1994/07/03  15:04:52  fjc
  - Incorporated changes in 3.1 Interfaces

  Revision 1.5  1994/06/14  02:14:31  fjc
  - Updated for release

  Revision 1.4  1994/06/09  14:13:42  fjc
  - Incorporates changes to Amiga interface

  Revision 1.3  1994/06/04  16:03:39  fjc
  - Changed to use new Amiga interface

  Revision 1.2  1994/05/12  20:45:18  fjc
  - Prepared for release

# Revision 1.1  1994/01/15  21:39:12  fjc
# Start of revision control
#
***************************************************************************)

